package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.TimSort;
import de.uni_marburg.powersort.data.Data1;
import de.uni_marburg.powersort.data.ObjectSupplier;

import java.util.Arrays;
import java.util.List;
import java.util.function.Supplier;

/**
 * Custom benchmark.
 */
public class Main {
    public static void main(final String[] args) {
        final SortImpl[] sortImplementations = getSortImplementations();
        final List<Supplier<ObjectSupplier>> sortInputSuppliers = getSortInputSuppliers();

        for (Supplier<ObjectSupplier> sortInputSupplier : sortInputSuppliers) {
            ObjectSupplier objectSupplier = sortInputSupplier.get();
            System.out.println("\n" + objectSupplier.title());

            for (SortImpl sortImplementation : sortImplementations) {
                Object[] sortInput = objectSupplier.get();

                final long startTime = System.nanoTime();
                sortImplementation.sort(sortInput);
                final long stopTime = System.nanoTime();
                final long duration = stopTime - startTime;

                System.out.println(duration + "," + sortImplementation.title);
            }
        }
    }

    static SortImpl[] getSortImplementations() {
        return new SortImpl[]{
                new SortImpl("DummySort") {
                    @Override
                    public void sort(Object[] a) {
                        DummySort.sort(a, 0, a.length, NaturalOrder.INSTANCE, null, 0, 0);
                    }
                },
                new SortImpl("TimSort") {
                    @Override
                    public void sort(Object[] a) {
                        TimSort.sort(a, 0, a.length, NaturalOrder.INSTANCE, null, 0, 0);
                    }
                },
                new SortImpl("MergeSort") {
                    @Override
                    public void sort(Object[] a) {
                        MergeSort.legacyMergeSort(a, NaturalOrder.INSTANCE);
                    }
                }
        };
    }

    static List<Supplier<ObjectSupplier>> getSortInputSuppliers() {
        return Arrays.asList(
                // Three different random lists.
                Data1::new,
                Data1::new,
                Data1::new
        );
    }
}
